@@ -173,7 +173,7 @@ void Cache::swap_index(int i, int j)
 //
 class Kernel {
 public:
-	Kernel(int l, const svm_node * const * x, const svm_parameter& param);
+	Kernel(int l, svm_node * const * x, const svm_parameter& param);
 	virtual ~Kernel();
 
 	static double k_function(const svm_node *x, const svm_node *y,
@@ -217,7 +217,7 @@ private:
 	}
 };
 
-Kernel::Kernel(int l, const svm_node * const * x_, const svm_parameter& param)
+Kernel::Kernel(int l, svm_node * const * x_, const svm_parameter& param)
 :kernel_type(param.kernel_type), degree(param.degree),
  gamma(param.gamma), coef0(param.coef0)
 {
@@ -235,9 +235,6 @@ Kernel::Kernel(int l, const svm_node * const * x_, const svm_parameter& param)
 		case SIGMOID:
 			kernel_function = &Kernel::kernel_sigmoid;
 			break;
-		default:
-			fprintf(stderr,"unknown kernel function.\n");
-			exit(1);
 	}
 
 	clone(x,x_,l);
@@ -332,11 +329,10 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 		}
 		case SIGMOID:
 			return tanh(param.gamma*dot(x,y)+param.coef0);
-		default:
-			break;
 	}
-	fprintf(stderr,"unknown kernel function.\n");
-	exit(1);
+	// This error condition should never occur because checked before by svn_check_parameters
+	//	fprintf(stderr,"unknown kernel function.\n");
+	//	exit(1);
 }
 
 // Generalized SMO+SVMlight algorithm
@@ -1297,27 +1293,13 @@ static void solve_nu_svc(
 	int l = prob->l;
 	double nu = param->nu;
 
-	int y_pos = 0;
-	int y_neg = 0;
 	schar *y = new schar[l];
 
 	for(i=0;i<l;i++)
 		if(prob->y[i]>0)
-		{
 			y[i] = +1;
-			++y_pos;
-		}
 		else
-		{
 			y[i] = -1;
-			++y_neg;
-		}
-
-	if(nu < 0 || nu*l/2 > min(y_pos,y_neg))
-	{
-		fprintf(stderr,"specified nu is infeasible\n");
-		exit(1);
-	}
 
 	double sum_pos = nu*l/2;
 	double sum_neg = nu*l/2;
@@ -1368,11 +1350,7 @@ static void solve_one_class(
 	int i;
 
 	int n = (int)(param->nu*prob->l);	// # of alpha's at upper bound
-	if(n>=prob->l)
-	{
-		fprintf(stderr,"nu must be in (0,1)\n");
-		exit(1);
-	}
+
 	for(i=0;i<n;i++)
 		alpha[i] = 1;
 	alpha[n] = param->nu * prob->l - n;
@@ -1435,12 +1413,6 @@ static void solve_nu_svr(
 	const svm_problem *prob, const svm_parameter *param,
 	double *alpha, Solver::SolutionInfo* si)
 {
-	if(param->nu < 0 || param->nu > 1)
-	{
-		fprintf(stderr,"specified nu is out of range\n");
-		exit(1);
-	}
-
 	int l = prob->l;
 	double C = param->C;
 	double *alpha2 = new double[2*l];
@@ -1670,9 +1642,10 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 			for(j=0;j<nr_class;j++)
 				if(param->weight_label[i] == label[j])
 					break;
-			if(j == nr_class)
-				fprintf(stderr,"warning: class label %d specified in weight is not found\n", param->weight_label[i]);
-			else
+			//			if(j == nr_class)
+			//				fprintf(stderr,"warning: class label %d specified in weight is not found\n", param->weight_label[i]);
+			//			else
+			if(j < nr_class)
 				weighted_C[j] *= param->weight[i];
 		}
 
@@ -1913,7 +1886,7 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 		fprintf(fp, "\n");
 	}
 	
-	if(model->label)
+ 	if(model->label)
 	{
 		fprintf(fp, "label");
 		for(int i=0;i<nr_class;i++)
@@ -1958,8 +1931,9 @@ svm_model *svm_load_model(const char *model_file_name)
 	
 	// read parameters
 
-	svm_model *model = (svm_model *)malloc(sizeof(svm_model));
+	svm_model *model = Malloc(svm_model,1);
 	svm_parameter& param = model->param;
+	model->rho = NULL;
 	model->label = NULL;
 	model->nSV = NULL;
 
@@ -1983,7 +1957,11 @@ svm_model *svm_load_model(const char *model_file_name)
 			if(svm_type_table[i] == NULL)
 			{
 				fprintf(stderr,"unknown svm type.\n");
-				exit(1);
+				free(model->rho);
+				free(model->label);
+				free(model->nSV);
+				free(model);
+				return NULL;
 			}
 		}
 		else if(strcmp(cmd,"kernel_type")==0)
@@ -2001,7 +1979,11 @@ svm_model *svm_load_model(const char *model_file_name)
 			if(kernel_type_table[i] == NULL)
 			{
 				fprintf(stderr,"unknown kernel function.\n");
-				exit(1);
+				free(model->rho);
+				free(model->label);
+				free(model->nSV);
+				free(model);
+				return NULL;
 			}
 		}
 		else if(strcmp(cmd,"degree")==0)
@@ -2047,7 +2029,11 @@ svm_model *svm_load_model(const char *model_file_name)
 		else
 		{
 			fprintf(stderr,"unknown text in model file\n");
-			exit(1);
+			free(model->rho);
+			free(model->label);
+			free(model->nSV);
+			free(model);
+			return NULL;
 		}
 	}
 
@@ -2124,3 +2110,107 @@ void svm_destroy_model(svm_model* model)
 	free(model->nSV);
 	free(model);
 }
+
+const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *param)
+{
+	// svm_type
+
+	int svm_type = param->svm_type;
+	if(svm_type != C_SVC &&
+	   svm_type != NU_SVC &&
+	   svm_type != ONE_CLASS &&
+	   svm_type != EPSILON_SVR &&
+	   svm_type != NU_SVR)
+		return "unknown svm type";
+	
+	// kernel_type
+	
+	int kernel_type = param->kernel_type;
+	if(kernel_type != LINEAR &&
+	   kernel_type != POLY &&
+	   kernel_type != RBF &&
+	   kernel_type != SIGMOID)
+		return "unknown kernel type";
+
+	// cache_size,eps,C,nu,p,shrinking
+
+	if(param->cache_size <= 0)
+		return "cache_size <= 0";
+
+	if(param->eps <= 0)
+		return "eps <= 0";
+
+	if(svm_type == C_SVC ||
+	   svm_type == EPSILON_SVR ||
+	   svm_type == NU_SVR)
+		if(param->C <= 0)
+			return "C <= 0";
+
+	if(svm_type == NU_SVC ||
+	   svm_type == ONE_CLASS ||
+	   svm_type == NU_SVR)
+		if(param->nu < 0 || param->nu > 1)
+			return "nu < 0 or nu > 1";
+
+	if(svm_type == EPSILON_SVR)
+		if(param->p < 0)
+			return "p < 0";
+
+	if(param->shrinking != 0 &&
+	   param->shrinking != 1)
+		return "shrinking != 0 and shrinking != 1";
+
+
+	// check whether nu-svc is feasible
+	
+	if(svm_type == NU_SVC)
+	{
+		int l = prob->l;
+		int max_nr_class = 16;
+		int nr_class = 0;
+		int *label = Malloc(int,max_nr_class);
+		int *count = Malloc(int,max_nr_class);
+
+		int i;
+		for(i=0;i<l;i++)
+		{
+			int this_label = (int)prob->y[i];
+			int j;
+			for(j=0;j<nr_class;j++)
+				if(this_label == label[j])
+				{
+					++count[j];
+					break;
+				}
+			if(j == nr_class)
+			{
+				if(nr_class == max_nr_class)
+				{
+					max_nr_class *= 2;
+					label = (int *)realloc(label,max_nr_class*sizeof(int));
+					count = (int *)realloc(count,max_nr_class*sizeof(int));
+				}
+				label[nr_class] = this_label;
+				count[nr_class] = 1;
+				++nr_class;
+			}
+		}
+	
+		for(i=0;i<nr_class;i++)
+		{
+			int n1 = count[i];
+			for(int j=i+1;j<nr_class;j++)
+			{
+				int n2 = count[j];
+				if(param->nu*(n1+n2)/2 > min(n1,n2))
+				{
+					free(label);
+					free(count);
+					return "specified nu is infeasible";
+				}
+			}
+		}
+	}
+
+	return NULL;
+}
